<?php

declare(strict_types=1);
namespace Drupal\confirmation\Cron;

use Drupal\confirmation\Entity\Confirmation;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

final class ConfirmationCron implements ConfirmationCronInterface {

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    protected int $limit
  ) {}

  public function hookCron(): void {
    $itemsToProcess = $this->limit;
    foreach ($this->entityTypeBundleInfo->getBundleInfo('confirmation') as $bundle => $bundleInfo) {
      /** @var \Drupal\confirmation\Entity\Confirmation $bundleClass */
      $bundleClass = $this->entityTypeManager->getStorage('confirmation')
        ->getEntityClass($bundle);
      $ids = $bundleClass::getEntityIdsToPurge($bundle, $itemsToProcess);
      foreach ($ids as $id) {
        Confirmation::load($id)->delete();
      }
      $itemsToProcess -= count($ids);
      if ($itemsToProcess <= 0) {
        break;
      }
    }
  }

}
