<?php

declare(strict_types=1);
namespace Drupal\confirmation\Entity\Event;

use Drupal\Component\EventDispatcher\Event;
use Drupal\confirmation\Entity\ConfirmationInterface;

final class ConfirmationEvent extends Event {

  protected ConfirmationInterface $confirmation;

  public function __construct(ConfirmationInterface $confirmation) {
    $this->confirmation = $confirmation;
  }

  public static function getEventName(string $bundle = NULL, bool $state = NULL) {
    if (isset($state) && !isset($bundle)) {
      throw new \InvalidArgumentException('There is no event with state but no bundle.');
    }
    $parts[] = 'confirmation';
    if (isset($bundle)) {
      $parts[] = $bundle;
    }
    $parts[] = 'state_settled';
    if (isset($state)) {
      $parts[] = $state ? 'true' : 'false';
    }
    return implode('.', $parts);
  }

  /**
   * @return list<string>
   */
  public function getEventNamesToDispatch(): array {
    $bundle = $this->confirmation->bundle();
    $state = $this->confirmation->getState();
    if (!isset($state)) {
      throw new \UnexpectedValueException('State must be set.');
    }
    return [
      // "confirmation.{$bundle}.state_settled.{$state}",
      self::getEventName($bundle, $state),
      // "confirmation.{$bundle}.state_settled",
      self::getEventName($bundle),
      // 'confirmation.state_settled',
      self::getEventName(),
    ];
  }

  public function getConfirmation(): ConfirmationInterface {
    return $this->confirmation;
  }

}
